@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -57,47 +49,14 @@
 
 QT_BEGIN_NAMESPACE
 
-static const char vertexShaderCode[] =
-    "attribute highp vec2 vPos;\n"
-    "attribute highp vec2 vTex;\n"
-    "uniform highp vec3 animData;// w,h(premultiplied of anim), interpolation progress\n"
-    "uniform highp vec4 animPos;//x,y, x,y (two frames for interpolation)\n"
-    "\n"
-    "uniform highp mat4 qt_Matrix;\n"
-    "\n"
-    "varying highp vec4 fTexS;\n"
-    "varying lowp float progress;\n"
-    "\n"
-    "\n"
-    "void main() {\n"
-    "    progress = animData.z;\n"
-    "    //Calculate frame location in texture\n"
-    "    fTexS.xy = animPos.xy + vTex.xy * animData.xy;\n"
-    "    //Next frame is also passed, for interpolation\n"
-    "    fTexS.zw = animPos.zw + vTex.xy * animData.xy;\n"
-    "\n"
-    "    gl_Position = qt_Matrix * vec4(vPos.x, vPos.y, 0, 1);\n"
-    "}\n";
-
-static const char fragmentShaderCode[] =
-    "uniform sampler2D texture;\n"
-    "uniform lowp float qt_Opacity;\n"
-    "\n"
-    "varying highp vec4 fTexS;\n"
-    "varying lowp float progress;\n"
-    "\n"
-    "void main() {\n"
-    "    gl_FragColor = mix(texture2D(texture, fTexS.xy), texture2D(texture, fTexS.zw), progress) * qt_Opacity;\n"
-    "}\n";
-
 class QQuickAnimatedSpriteMaterial : public QSGMaterial
 {
 public:
     QQuickAnimatedSpriteMaterial();
     ~QQuickAnimatedSpriteMaterial();
-    virtual QSGMaterialType *type() const { static QSGMaterialType type; return &type; }
-    virtual QSGMaterialShader *createShader() const;
-    virtual int compare(const QSGMaterial *other) const
+    QSGMaterialType *type() const Q_DECL_OVERRIDE { static QSGMaterialType type; return &type; }
+    QSGMaterialShader *createShader() const Q_DECL_OVERRIDE;
+    int compare(const QSGMaterial *other) const Q_DECL_OVERRIDE
     {
         return this - static_cast<const QQuickAnimatedSpriteMaterial *>(other);
     }
@@ -134,19 +93,14 @@ QQuickAnimatedSpriteMaterial::~QQuickAnimatedSpriteMaterial()
 class AnimatedSpriteMaterialData : public QSGMaterialShader
 {
 public:
-    AnimatedSpriteMaterialData(const char * /* vertexFile */ = 0, const char * /* fragmentFile */ = 0)
+    AnimatedSpriteMaterialData()
+        : QSGMaterialShader()
     {
+        setShaderSourceFile(QOpenGLShader::Vertex, QStringLiteral(":/qt-project.org/items/shaders/sprite.vert"));
+        setShaderSourceFile(QOpenGLShader::Fragment, QStringLiteral(":/qt-project.org/items/shaders/sprite.frag"));
     }
 
-    void deactivate() {
-        QSGMaterialShader::deactivate();
-
-        for (int i=0; i<8; ++i) {
-            program()->setAttributeArray(i, GL_FLOAT, chunkOfBytes, 1, 0);
-        }
-    }
-
-    virtual void updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *)
+    void updateState(const RenderState &state, QSGMaterial *newEffect, QSGMaterial *) Q_DECL_OVERRIDE
     {
         QQuickAnimatedSpriteMaterial *m = static_cast<QQuickAnimatedSpriteMaterial *>(newEffect);
         m->texture->bind();
@@ -159,17 +113,14 @@ public:
             program()->setUniformValue(m_matrix_id, state.combinedMatrix());
     }
 
-    virtual void initialize() {
+    void initialize() Q_DECL_OVERRIDE {
         m_matrix_id = program()->uniformLocation("qt_Matrix");
         m_opacity_id = program()->uniformLocation("qt_Opacity");
         m_animData_id = program()->uniformLocation("animData");
         m_animPos_id = program()->uniformLocation("animPos");
     }
 
-    virtual const char *vertexShader() const { return vertexShaderCode; }
-    virtual const char *fragmentShader() const { return fragmentShaderCode; }
-
-    virtual char const *const *attributeNames() const {
+    char const *const *attributeNames() const Q_DECL_OVERRIDE {
         static const char *attr[] = {
            "vPos",
            "vTex",
@@ -182,12 +133,8 @@ public:
     int m_opacity_id;
     int m_animData_id;
     int m_animPos_id;
-
-    static float chunkOfBytes[1024];
 };
 
-float AnimatedSpriteMaterialData::chunkOfBytes[1024];
-
 QSGMaterialShader *QQuickAnimatedSpriteMaterial::createShader() const
 {
     return new AnimatedSpriteMaterialData;
@@ -210,7 +157,7 @@ struct AnimatedSpriteVertices {
 /*!
     \qmltype AnimatedSprite
     \instantiates QQuickAnimatedSprite
-    \inqmlmodule QtQuick 2
+    \inqmlmodule QtQuick
     \inherits Item
     \ingroup qtquick-visual
     \brief Draws a sprite animation
@@ -219,13 +166,13 @@ struct AnimatedSpriteVertices {
     as multiple frames in the same image file. You can play it at a fixed speed, at the
     frame rate of your display, or manually advance and control the progress.
 
-    For details of how a sprite animation is defined see the \l{Sprite Animation} overview.
+    For details of how a sprite animation is defined see the \l{Sprite Animations} overview.
     Note that the AnimatedSprite type does not use Sprite types to define multiple animations,
     but instead encapsulates a single animation itself.
 */
 
 /*!
-    \qmlproperty bool QtQuick2::AnimatedSprite::running
+    \qmlproperty bool QtQuick::AnimatedSprite::running
 
     Whether the sprite is animating or not.
 
@@ -233,7 +180,7 @@ struct AnimatedSpriteVertices {
 */
 
 /*!
-    \qmlproperty bool QtQuick2::AnimatedSprite::interpolate
+    \qmlproperty bool QtQuick::AnimatedSprite::interpolate
 
     If true, interpolation will occur between sprite frames to make the
     animation appear smoother.
@@ -242,7 +189,7 @@ struct AnimatedSpriteVertices {
 */
 
 /*!
-    \qmlproperty qreal QtQuick2::AnimatedSprite::frameRate
+    \qmlproperty qreal QtQuick::AnimatedSprite::frameRate
 
     Frames per second to show in the animation. Values equal to or below 0 are invalid.
 
@@ -253,7 +200,7 @@ struct AnimatedSpriteVertices {
 */
 
 /*!
-    \qmlproperty int QtQuick2::AnimatedSprite::frameDuration
+    \qmlproperty int QtQuick::AnimatedSprite::frameDuration
 
     Duration of each frame of the animation. Values equal to or below 0 are invalid.
 
@@ -264,40 +211,40 @@ struct AnimatedSpriteVertices {
 */
 
 /*!
-    \qmlproperty int QtQuick2::AnimatedSprite::frameCount
+    \qmlproperty int QtQuick::AnimatedSprite::frameCount
 
     Number of frames in this AnimatedSprite.
 */
 /*!
-    \qmlproperty int QtQuick2::AnimatedSprite::frameHeight
+    \qmlproperty int QtQuick::AnimatedSprite::frameHeight
 
     Height of a single frame in this AnimatedSprite.
 
     May be omitted if it is the only sprite in the file.
 */
 /*!
-    \qmlproperty int QtQuick2::AnimatedSprite::frameWidth
+    \qmlproperty int QtQuick::AnimatedSprite::frameWidth
 
     Width of a single frame in this AnimatedSprite.
 
     May be omitted if it is the only sprite in the file.
 */
 /*!
-    \qmlproperty int QtQuick2::AnimatedSprite::frameX
+    \qmlproperty int QtQuick::AnimatedSprite::frameX
 
     The X coordinate in the image file of the first frame of the AnimatedSprite.
 
     May be omitted if the first frame starts in the upper left corner of the file.
 */
 /*!
-    \qmlproperty int QtQuick2::AnimatedSprite::frameY
+    \qmlproperty int QtQuick::AnimatedSprite::frameY
 
     The Y coordinate in the image file of the first frame of the AnimatedSprite.
 
     May be omitted if the first frame starts in the upper left corner of the file.
 */
 /*!
-    \qmlproperty url QtQuick2::AnimatedSprite::source
+    \qmlproperty url QtQuick::AnimatedSprite::source
 
     The image source for the animation.
 
@@ -308,7 +255,7 @@ struct AnimatedSpriteVertices {
 */
 
 /*!
-    \qmlproperty bool QtQuick2::AnimatedSprite::reverse
+    \qmlproperty bool QtQuick::AnimatedSprite::reverse
 
     If true, then the animation will be played in reverse.
 
@@ -316,7 +263,7 @@ struct AnimatedSpriteVertices {
 */
 
 /*!
-    \qmlproperty bool QtQuick2::AnimatedSprite::frameSync
+    \qmlproperty bool QtQuick::AnimatedSprite::frameSync
 
     If true, then the animation will have no duration. Instead, the animation will advance
     one frame each time a frame is rendered to the screen. This synchronizes it with the painting
@@ -330,7 +277,7 @@ struct AnimatedSpriteVertices {
 */
 
 /*!
-    \qmlproperty int QtQuick2::AnimatedSprite::loops
+    \qmlproperty int QtQuick::AnimatedSprite::loops
 
     After playing the animation this many times, the animation will automatically stop. Negative values are invalid.
 
@@ -340,7 +287,7 @@ struct AnimatedSpriteVertices {
 */
 
 /*!
-    \qmlproperty bool QtQuick2::AnimatedSprite::paused
+    \qmlproperty bool QtQuick::AnimatedSprite::paused
 
     When paused, the current frame can be advanced manually.
 
@@ -348,14 +295,14 @@ struct AnimatedSpriteVertices {
 */
 
 /*!
-    \qmlproperty int QtQuick2::AnimatedSprite::currentFrame
+    \qmlproperty int QtQuick::AnimatedSprite::currentFrame
 
     When paused, the current frame can be advanced manually by setting this property or calling advance().
 
 */
 
 /*!
-    \qmlmethod int QtQuick2::AnimatedSprite::restart()
+    \qmlmethod int QtQuick::AnimatedSprite::restart()
 
     Stops, then starts the sprite animation.
 */
@@ -363,8 +310,6 @@ struct AnimatedSpriteVertices {
 //TODO: Implicitly size element to size of sprite
 QQuickAnimatedSprite::QQuickAnimatedSprite(QQuickItem *parent) :
     QQuickItem(parent)
-    , m_node(0)
-    , m_material(0)
     , m_sprite(new QQuickSprite(this))
     , m_spriteEngine(0)
     , m_curFrame(0)
@@ -377,12 +322,10 @@ QQuickAnimatedSprite::QQuickAnimatedSprite(QQuickItem *parent) :
     , m_pauseOffset(0)
 {
     setFlag(ItemHasContents);
-    connect(this, SIGNAL(runningChanged(bool)),
-            this, SLOT(update()));
     connect(this, SIGNAL(widthChanged()),
-            this, SLOT(sizeVertices()));
+            this, SLOT(reset()));
     connect(this, SIGNAL(heightChanged()),
-            this, SLOT(sizeVertices()));
+            this, SLOT(reset()));
 }
 
 bool QQuickAnimatedSprite::isCurrentFrameChangedConnected()
@@ -429,10 +372,11 @@ void QQuickAnimatedSprite::stop()
         return;
     m_pauseOffset = 0;
     emit runningChanged(false);
+    update();
 }
 
 /*!
-    \qmlmethod int QtQuick2::AnimatedSprite::advance()
+    \qmlmethod int QtQuick::AnimatedSprite::advance()
 
     Advances the sprite animation by one frame.
 */
@@ -443,13 +387,14 @@ void QQuickAnimatedSprite::advance(int frames)
     //TODO-C: May not work when running - only when paused
     m_curFrame += frames;
     while (m_curFrame < 0)
-        m_curFrame += m_sprite->frames();
-    m_curFrame = m_curFrame % m_sprite->frames();
+        m_curFrame += m_spriteEngine->maxFrames();
+    m_curFrame = m_curFrame % m_spriteEngine->maxFrames();
     emit currentFrameChanged(m_curFrame);
+    update();
 }
 
 /*!
-    \qmlmethod int QtQuick2::AnimatedSprite::pause()
+    \qmlmethod int QtQuick::AnimatedSprite::pause()
 
     Pauses the sprite animation. This does nothing if
     \l paused is true.
@@ -463,10 +408,11 @@ void QQuickAnimatedSprite::pause()
     m_pauseOffset = m_timestamp.elapsed();
     m_paused = true;
     emit pausedChanged(true);
+    update();
 }
 
 /*!
-    \qmlmethod int QtQuick2::AnimatedSprite::resume()
+    \qmlmethod int QtQuick::AnimatedSprite::resume()
 
     Resumes the sprite animation if \l paused is true;
     otherwise, this does nothing.
@@ -480,6 +426,7 @@ void QQuickAnimatedSprite::resume()
     m_pauseOffset = m_pauseOffset - m_timestamp.elapsed();
     m_paused = false;
     emit pausedChanged(false);
+    update();
 }
 
 void QQuickAnimatedSprite::createEngine()
@@ -491,6 +438,7 @@ void QQuickAnimatedSprite::createEngine()
     m_spriteEngine = new QQuickSpriteEngine(QList<QQuickSprite*>(spriteList), this);
     m_spriteEngine->startAssemblingImage();
     reset();
+    update();
 }
 
 static QSGGeometry::Attribute AnimatedSprite_Attributes[] = {
@@ -505,12 +453,9 @@ static QSGGeometry::AttributeSet AnimatedSprite_AttributeSet =
     AnimatedSprite_Attributes
 };
 
-void QQuickAnimatedSprite::sizeVertices()
+void QQuickAnimatedSprite::sizeVertices(QSGGeometryNode *node)
 {
-    if (!m_node)
-        return;
-
-    AnimatedSpriteVertices *p = (AnimatedSpriteVertices *) m_node->geometry()->vertexData();
+    AnimatedSpriteVertices *p = (AnimatedSpriteVertices *) node->geometry()->vertexData();
     p->v1.x = 0;
     p->v1.y = 0;
 
@@ -538,22 +483,21 @@ QSGGeometryNode* QQuickAnimatedSprite::buildNode()
         return 0;
     }
 
-    m_material = new QQuickAnimatedSpriteMaterial();
+    QQuickAnimatedSpriteMaterial *material = new QQuickAnimatedSpriteMaterial();
 
     QImage image = m_spriteEngine->assembledImage(); //Engine prints errors if there are any
     if (image.isNull())
         return 0;
     m_sheetSize = QSizeF(image.size());
-    m_material->texture = window()->createTextureFromImage(image);
-    m_material->texture->setFiltering(QSGTexture::Linear);
+    material->texture = window()->createTextureFromImage(image);
     m_spriteEngine->start(0);
-    m_material->animT = 0;
-    m_material->animX1 = m_spriteEngine->spriteX() / m_sheetSize.width();
-    m_material->animY1 = m_spriteEngine->spriteY() / m_sheetSize.height();
-    m_material->animX2 = m_material->animX1;
-    m_material->animY2 = m_material->animY1;
-    m_material->animW = m_spriteEngine->spriteWidth() / m_sheetSize.width();
-    m_material->animH = m_spriteEngine->spriteHeight() / m_sheetSize.height();
+    material->animT = 0;
+    material->animX1 = m_spriteEngine->spriteX() / m_sheetSize.width();
+    material->animY1 = m_spriteEngine->spriteY() / m_sheetSize.height();
+    material->animX2 = material->animX1;
+    material->animY2 = material->animY1;
+    material->animW = m_spriteEngine->spriteWidth() / m_sheetSize.width();
+    material->animH = m_spriteEngine->spriteHeight() / m_sheetSize.height();
 
     int vCount = 4;
     int iCount = 6;
@@ -562,7 +506,7 @@ QSGGeometryNode* QQuickAnimatedSprite::buildNode()
 
     AnimatedSpriteVertices *p = (AnimatedSpriteVertices *) g->vertexData();
 
-    QRectF texRect = m_material->texture->normalizedTextureSubRect();
+    QRectF texRect = material->texture->normalizedTextureSubRect();
 
     p->v1.tx = texRect.topLeft().x();
     p->v1.ty = texRect.topLeft().y();
@@ -585,106 +529,175 @@ QSGGeometryNode* QQuickAnimatedSprite::buildNode()
     indices[5] = 2;
 
 
-    m_timestamp.start();
-    m_node = new QSGGeometryNode();
-    m_node->setGeometry(g);
-    m_node->setMaterial(m_material);
-    m_node->setFlag(QSGGeometryNode::OwnsMaterial);
-    sizeVertices();
-    return m_node;
+    QSGGeometryNode *node = new QSGGeometryNode();
+    node->setGeometry(g);
+    node->setMaterial(material);
+    node->setFlag(QSGGeometryNode::OwnsMaterial);
+    node->setFlag(QSGGeometryNode::OwnsGeometry);
+    sizeVertices(node);
+    return node;
 }
 
 void QQuickAnimatedSprite::reset()
 {
     m_pleaseReset = true;
+    update();
 }
 
-QSGNode *QQuickAnimatedSprite::updatePaintNode(QSGNode *, UpdatePaintNodeData *)
+QSGNode *QQuickAnimatedSprite::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *)
 {
     if (m_pleaseReset) {
-        delete m_node;
+        delete oldNode;
 
-        m_node = 0;
-        m_material = 0;
+        oldNode = 0;
         m_pleaseReset = false;
     }
 
-    prepareNextFrame();
+    QSGGeometryNode *node = static_cast<QSGGeometryNode *>(oldNode);
+    if (!node)
+        node = buildNode();
+
+    if (node)
+        prepareNextFrame(node);
 
     if (m_running) {
-        update();
-        if (m_node)
-            m_node->markDirty(QSGNode::DirtyMaterial);
+        if (!m_paused)
+            update();
+
+        if (node) {
+            node->markDirty(QSGNode::DirtyMaterial);
+        }
     }
 
-    return m_node;
+    return node;
 }
 
-void QQuickAnimatedSprite::prepareNextFrame()
+void QQuickAnimatedSprite::prepareNextFrame(QSGGeometryNode *node)
 {
-    if (m_node == 0)
-        m_node = buildNode();
-    if (m_node == 0) //error creating node
-        return;
-
     int timeInt = m_timestamp.elapsed() + m_pauseOffset;
     qreal time =  timeInt / 1000.;
 
-    double frameAt; //double just for modf
+    int frameAt;
     qreal progress = 0.0;
     int lastFrame = m_curFrame;
-    if (!m_paused) {
+    if (m_running && !m_paused) {
+        const int nColumns = int(m_sheetSize.width()) / m_spriteEngine->spriteWidth();
         //Advance State (keeps time for psuedostates)
         m_spriteEngine->updateSprites(timeInt);
 
         //Advance AnimatedSprite
         qreal animT = m_spriteEngine->spriteStart()/1000.0;
-        qreal frameCount = m_spriteEngine->spriteFrames();
-        qreal frameDuration = m_spriteEngine->spriteDuration()/frameCount;
+        const int frameCountInRow = m_spriteEngine->spriteFrames();
+        const qreal frameDuration = m_spriteEngine->spriteDuration()/frameCountInRow;
         if (frameDuration > 0) {
             qreal frame = (time - animT)/(frameDuration / 1000.0);
-            frame = qBound(qreal(0.0), frame, frameCount - qreal(1.0));//Stop at count-1 frames until we have between anim interpolation
-            progress = modf(frame,&frameAt);
-            if (m_curFrame > frameAt) //went around
+            bool lastLoop = m_loops > 0 && m_curLoop == m_loops-1;
+            //don't visually interpolate for the last frame of the last loop
+            const int max = lastLoop ? frameCountInRow - 1 : frameCountInRow;
+            frame = qBound(qreal(0.0), frame, qreal(max));
+            double intpart;
+            progress = std::modf(frame,&intpart);
+            frameAt = (int)intpart;
+            const int rowIndex = m_spriteEngine->spriteY()/frameHeight();
+            const int newFrame = rowIndex * nColumns + frameAt;
+            if (m_curFrame > newFrame) //went around
                 m_curLoop++;
-            m_curFrame = frameAt;
+            m_curFrame = newFrame;
         } else {
             m_curFrame++;
-            if (m_curFrame >= frameCount){
+            if (m_curFrame >= m_spriteEngine->maxFrames()) {    // maxFrames: total number of frames including all rows
                 m_curFrame = 0;
                 m_curLoop++;
-                m_spriteEngine->advance();
             }
-            frameAt = m_curFrame;
+            frameAt = m_curFrame % nColumns;
+            if (frameAt == 0)
+                m_spriteEngine->advance();
             progress = 0;
         }
         if (m_loops > 0 && m_curLoop >= m_loops) {
             frameAt = 0;
             m_running = false;
+            emit runningChanged(false);
+            update();
         }
     } else {
         frameAt = m_curFrame;
     }
-    if (m_curFrame != lastFrame && isCurrentFrameChangedConnected())
-        emit currentFrameChanged(m_curFrame);
-    if (m_spriteEngine->sprite()->reverse())
-        frameAt = (m_spriteEngine->spriteFrames() - 1) - frameAt;
-    qreal y = m_spriteEngine->spriteY() / m_sheetSize.height();
+    if (m_curFrame != lastFrame) {
+        if (isCurrentFrameChangedConnected())
+            emit currentFrameChanged(m_curFrame);
+        update();
+    }
+
+    qreal frameCount = m_spriteEngine->spriteFrames();
+    bool reverse = m_spriteEngine->sprite()->reverse();
+    if (reverse)
+        frameAt = (frameCount - 1) - frameAt;
+
     qreal w = m_spriteEngine->spriteWidth() / m_sheetSize.width();
     qreal h = m_spriteEngine->spriteHeight() / m_sheetSize.height();
-    qreal x1 = m_spriteEngine->spriteX() / m_sheetSize.width();
-    x1 += frameAt * w;
-    qreal x2 = x1;
-    if (frameAt < (m_spriteEngine->spriteFrames()-1))
-        x2 += w;
+    qreal x1;
+    qreal y1;
+    if (m_paused) {
+        int spriteY = m_spriteEngine->spriteY();
+        if (reverse) {
+            int rows = m_spriteEngine->maxFrames() * m_spriteEngine->spriteWidth() / m_sheetSize.width();
+            spriteY -= rows * m_spriteEngine->spriteHeight();
+            frameAt = (frameCount - 1) - frameAt;
+        }
 
-    m_material->animX1 = x1;
-    m_material->animY1 = y;
-    m_material->animX2 = x2;
-    m_material->animY2 = y;
-    m_material->animW = w;
-    m_material->animH = h;
-    m_material->animT = m_interpolate ? progress : 0.0;
+        int position = frameAt * m_spriteEngine->spriteWidth() + m_spriteEngine->spriteX();
+        int row = position / m_sheetSize.width();
+
+        x1 = (position - (row * m_sheetSize.width())) / m_sheetSize.width();
+        y1 = (row * m_spriteEngine->spriteHeight() + spriteY) / m_sheetSize.height();
+    } else {
+        x1 = m_spriteEngine->spriteX() / m_sheetSize.width() + frameAt * w;
+        y1 = m_spriteEngine->spriteY() / m_sheetSize.height();
+    }
+
+    //### hard-coded 0/1 work because we are the only
+    // images in the sprite sheet (without this we cannot assume
+    // where in the sheet we begin/end).
+    qreal x2;
+    qreal y2;
+    if (reverse) {
+        if (frameAt > 0) {
+            x2 = x1 - w;
+            y2 = y1;
+        } else {
+            x2 = 1.0 - w;
+            y2 = y1 - h;
+            if (y2 < 0.0) {
+                //the last row may not fill the entire width
+                int maxRowFrames = m_sheetSize.width() / m_spriteEngine->spriteWidth();
+                if (m_spriteEngine->maxFrames() % maxRowFrames)
+                    x2 = ((m_spriteEngine->maxFrames() % maxRowFrames) - 1) * w;
+
+                y2 = 1.0 - h;
+            }
+        }
+    } else {
+        if (frameAt < (frameCount-1)) {
+            x2 = x1 + w;
+            y2 = y1;
+        } else {
+            x2 = 0.0;
+            y2 = y1 + h;
+            if (y2 >= 1.0)
+                y2 = 0.0;
+        }
+    }
+
+    QQuickAnimatedSpriteMaterial *material = static_cast<QQuickAnimatedSpriteMaterial *>(node->material());
+    material->animX1 = x1;
+    material->animY1 = y1;
+    material->animX2 = x2;
+    material->animY2 = y2;
+    material->animW = w;
+    material->animH = h;
+    material->animT = m_interpolate ? progress : 0.0;
+    material->texture->setFiltering(smooth() ? QSGTexture::Linear : QSGTexture::Nearest);
 }
 
 QT_END_NAMESPACE
